<?php

namespace WHMCS\Module\Notification\Discord;

use WHMCS\Config\Setting;
use WHMCS\Exception;
use WHMCS\Mail\Template;
use WHMCS\Module\Notification\DescriptionTrait;
use WHMCS\Module\Contracts\NotificationModuleInterface;
use WHMCS\Notification\Contracts\NotificationInterface;
use WHMCS\Utility\Environment\WebHelper;
use \DateTime;

class Discord implements NotificationModuleInterface
{
    use DescriptionTrait;

    public function __construct()
    {
        $this->setDisplayName('Discord')
            ->setLogoFileName('discord.png');
    }

    public function settings()
    {
        return [
            'discordWebhook' => [
                'FriendlyName' => 'Webhook URL',
                'Type' => 'text',
                'Description' => 'The URL of the Discord Webhook the notification should be sent to. To create a new Discord Webhook, heads up to "Server Settings > Webhooks".',
                'Placeholder' => "https://discordapp.com/api/webhooks/123456789012345/kjbkjfsdjkbfsdkbfdsjksdfajksdfjkfsdajkjfadkjfd",
            ],
            'senderName' => [
                'FriendlyName' => 'Sender Name',
                'Type' => 'text',
                'Description' => 'The name the notification should come from. Overwrite the webhook name (optional)',
                'Placeholder' => Setting::getValue('CompanyName'),
            ],
            'discordColor' => [
                'FriendlyName' => 'Discord Embed Color',
                'Type' => 'text',
                'Description' => 'The hex color code of the embed. (without #, optional)',
                'Placeholder' => "7289da",
            ],
            'discordAvatar' => [
                'FriendlyName' => 'Discord Avatar',
                'Type' => 'text',
                'Description' => 'The image that should be the avatar of the webhook. Overwrite the webhook avatar. (optional)',
                'Placeholder' => "https://media.glassdoor.com/sqll/827249/whmcs-squarelogo-1469016322333.png",
            ],
        ];
    }

    public function testConnection($settings)
    {
        $postdata = array(
            'username' => $settings['senderName'],
            'avatar_url' => $settings['discordAvatar'],
            'embeds' => array(
                array(
                    'title' => 'This is a test.',
                    'timestamp' => date(DateTime::ISO8601),
                    'color' => hexdec($settings['discordColor']),
                    'description' => 'This is a test confirming that Discord Notifications Module is working.',
                    'author' => array(
                        'name' => 'Test',
                    )
                )
            )
        );
        try {
            $this->call($settings, $postdata);
        } catch (\WHMCS\Exception $e) {
            throw new \WHMCS\Exception($e->getMessage());
        }
    }

    public function notificationSettings()
    {
        return [

        ];
    }

    public function getDynamicField($fieldName, $settings)
    {
        return [];
    }

    public function sendNotification(NotificationInterface $notification, $moduleSettings, $notificationSettings)
    {
        $fields = [];
        foreach ($notification->getAttributes() as $attribute) {
            $value = $attribute->getValue();

            array_push($fields, [
                'name' => $attribute->getLabel(),
                'value' => $value,
                'inline' => true
            ]);
        }
        $postdata = array(
            'username' => $moduleSettings['senderName'],
            'avatar_url' => $moduleSettings['discordAvatar'],
            'embeds' => array(
                array(
                    'title' => $notification->getTitle(),
                    'url' => $notification->getUrl(),
                    'timestamp' => date(DateTime::ISO8601),
                    'description' => $notification->getMessage(),
                    'color' => hexdec($moduleSettings['discordColor']),
                    'author' => array(
                        'name' => $notification->getTitle(),
                    ),
                    'fields' => $fields
                )
            )
        );
        $this->call($moduleSettings, $postdata);
    }

    protected function call($settings, array $postdata = array())
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $settings['discordWebhook']);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Content-Type: application/json'
        ));
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($postdata));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, 10);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        $response = curl_exec($ch);
        if (curl_getinfo($ch, CURLINFO_HTTP_CODE) != 204) {
            throw new \WHMCS\Exception("Bad response: " . $response);
        }
        curl_close($ch);
        $decoded = json_decode($response);
        logModuleCall("discord", $settings['discordWebhook'], $postdata, $response, $decoded, array($settings["discordWebhook"]));
        return $decoded;
    }
}
